import matchit from '@poppinss/matchit';
import lodash from '@poppinss/utils/lodash';
import { RuntimeException } from '@poppinss/utils';
export class RoutesStore {
    usingDomains = false;
    tree = { tokens: [], domains: {} };
    #getDomainNode(domain) {
        if (!this.tree.domains[domain]) {
            this.tree.tokens.push(matchit.parse(domain));
            this.tree.domains[domain] = {};
        }
        return this.tree.domains[domain];
    }
    #getMethodNode(domain, method) {
        const domainNode = this.#getDomainNode(domain);
        if (!domainNode[method]) {
            domainNode[method] = { tokens: [], routes: {}, routeKeys: {} };
        }
        return domainNode[method];
    }
    #collectRouteParams(route, tokens) {
        const collectedParams = new Set();
        for (let token of tokens) {
            if ([1, 3].includes(token.type)) {
                if (collectedParams.has(token.val)) {
                    throw new RuntimeException(`Duplicate param "${token.val}" found in "${route.pattern}"`);
                }
                else {
                    collectedParams.add(token.val);
                }
            }
        }
        const params = [...collectedParams];
        collectedParams.clear();
        return params;
    }
    #registerRoute(domain, method, tokens, route) {
        const methodRoutes = this.#getMethodNode(domain, method);
        if (methodRoutes.routes[route.pattern]) {
            throw new RuntimeException(`Duplicate route found. "${method}: ${route.pattern}" route already exists`);
        }
        methodRoutes.tokens.push(tokens);
        methodRoutes.routes[route.pattern] = route;
        methodRoutes.routeKeys[route.pattern] =
            domain !== 'root' ? `${domain}-${method}-${route.pattern}` : `${method}-${route.pattern}`;
    }
    add(route) {
        if (route.domain !== 'root') {
            this.usingDomains = true;
        }
        const tokens = matchit.parse(route.pattern, route.matchers);
        const routeNode = lodash.merge({ meta: {} }, lodash.pick(route, ['pattern', 'handler', 'meta', 'middleware', 'name', 'execute']));
        routeNode.meta.params = this.#collectRouteParams(routeNode, tokens);
        route.methods.forEach((method) => {
            this.#registerRoute(route.domain, method, tokens, routeNode);
        });
        return this;
    }
    match(url, method, domain) {
        const domainName = domain?.tokens[0]?.old || 'root';
        const matchedDomain = this.tree.domains[domainName];
        if (!matchedDomain) {
            return null;
        }
        const matchedMethod = this.tree.domains[domainName][method];
        if (!matchedMethod) {
            return null;
        }
        const matchedRoute = matchit.match(url, matchedMethod.tokens);
        if (!matchedRoute.length) {
            return null;
        }
        const route = matchedMethod.routes[matchedRoute[0].old];
        return {
            route: route,
            routeKey: matchedMethod.routeKeys[route.pattern],
            params: matchit.exec(url, matchedRoute),
            subdomains: domain?.hostname ? matchit.exec(domain.hostname, domain.tokens) : {},
        };
    }
    matchDomain(hostname) {
        if (!hostname || !this.usingDomains) {
            return [];
        }
        return matchit.match(hostname, this.tree.tokens);
    }
}
